#!/usr/bin/env python
# dbapi.py
# Maintained by W. Yang & G. Winter
# 
# An interface layer to hide the differences between MySQL API versions
# for python, in particular v 0.9.x to 1.1.x. These interfaces may be
# extended to include more versions as needed, so long as the following
# API is provided:
# 
# execute(sql_command)
# fetchall()
# fetchone()
# insert_id()
# close()
# version()
#
# The connection is opened in the constructor. All new interface
# implementations should be registered with the DBConnection factory method
# below.
# 
# 
# $Id: dbapi.py,v 1.8 2005/08/25 09:28:08 wy45 Exp $
# 

import MySQLdb

class DBConnection_MySQL0:
    ''' A MySQL Connection for the API MySQLdb 1.0 '''
    def __init__(self,hostname,user,password,dbname):
        ''' Create a connection to a MySQL database, and provide a cursor''' 
        self.connection = MySQLdb.connect(host = hostname,
                                          user = user,
                                          passwd = password,
                                          db = dbname)
        self.cursor = self.connection.cursor()
    def execute(self,sql_statement):
        ''' Execute the mysql statement ''' 
        return self.cursor.execute(sql_statement)
    def fetchall(self):
        '''Fetch all of the results from the previous statement ''' 
        return self.cursor.fetchall()
    def fetchone(self):
        '''Fetch just one result from the previous statement ''' 
        return self.cursor.fetchone()
    def insert_id(self):
        '''Get the last insert id from last statement ''' 
        return self.cursor.insert_id()
    def close(self):
        ''' Close the connection''' 
        self.connection.close()
    def version(self):
        ''' Get the module version''' 
        return MySQLdb.version_info 
    def escape(self,s):
        ''' escape string '''
        return MySQLdb.escape_string(s)

class DBConnection_MySQL1:
    '''A MySQL Connection for the API MySQLdb 1.1+'''

    def __init__(self, hostname, user, password, dbname):
        '''Create a connection to a MySQL database, and provide a cursor'''

        self.connection = MySQLdb.connect(host = hostname,
                                          user = user,
                                          passwd = password,
                                          db = dbname)
        self.cursor = self.connection.cursor()

    def execute(self, statement):
        '''Execute the mysql statement'''
        return self.cursor.execute(statement)

    def fetchall(self):
        '''Fetch all of the results from the previous statement'''
        return self.cursor.fetchall()

    def fetchone(self):
        '''Fetch just one result from the previous statement'''
        return self.cursor.fetchone()

    def insert_id(self):
        '''Get the last insert id from the last statement'''
        return self.connection.insert_id()

    def close(self):
        '''Close the connection'''
        return self.connection.close()

    def version(self):
        ''' Get the module version '''
        return MySQLdb.version_info 

    def escape(self,s):
        ''' escape string '''
        return MySQLdb.escape_string(s)
    
        
def DBConnection(hostname, user, passwd, dbname):
    '''A factory to produce correct DB connection object based on the
    version of MySQL'''

    if MySQLdb.version_info[0] == 0:
        return DBConnection_MySQL0(hostname, user, passwd, dbname)
    else:
        return DBConnection_MySQL1(hostname, user, passwd, dbname)

def Version():
    return MySQLdb.version_info



if __name__ == '__main__':
    # we should have some kind of unit test here
    # here's an example

    db = DBConnection('localhost', 'admin', 'admin', 'ProjectHistory')

    # test fetchall
    db.execute('show tables')
    results = db.fetchall()

    print results

    # test fetchone
    db.execute('show tables')
    while True:
        results = db.fetchone()
        if not results:
            break
        print results

    db.close()
